# -*- coding: utf-8 -*-
import wx
import atled
from subprocess import Popen, PIPE

class MainFrame(wx.Frame):
    # Constants
    LED_RED_ID = 0
    LED_YELLOW_ID = 1
    KOTAI_ID_A640_ES = "0097"
    KOTAI_ID_A640 = "009C"
    KOTAI_ID_A610_ES = "00B4"
    KOTAI_ID_A610 = "00B7"

    # Constructor
    def __init__(self, kotai_id):
        wx.Frame.__init__(self, None, title="LED VIEW", size=(800, 480))
        self.kotai_id = kotai_id
        self.Center()

        self.__setup_menu_bar()
        self.__setup_panels()
        self.__setup_layouts()

        self.delegate = None
        self.led_status = {self.LED_RED_ID:False, self.LED_YELLOW_ID:False}
        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            self.led_images = ["./resources/a640_clear.png", "./resources/a640_r_on.png", "./resources/a640_y_on.png", "./resources/a640_ry_on.png"]
        else:
            self.led_images = ["./resources/a610_clear.png", "./resources/a610_clear.png", "./resources/a610_y_on.png", "./resources/a610_y_on.png"]
        self.sw_images = ["./resources/a640_clear.png", "./resources/a640_sw_on.png"]

    # Public methods
    def set_led_status(self, led_status):
        self.led_status[self.LED_RED_ID] = bool(led_status & 1)
        self.led_status[self.LED_YELLOW_ID] = bool(led_status & 2)
        self.__update_led_image()

    # Callbacks
    def menu_selected(self, event):
        if self.delegate is not None:
            self.delegate.menu_selected(self, event.GetId())

    def button_pressed(self, event):
        id = event.GetId()
        self.led_status[id] = not self.led_status[id]
        self.__update_led_image()
        if self.delegate is not None:
            self.delegate.button_pressed(self, id, self.led_status[id])
    
    def key_pressed(self, event):
        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            key_code = event.GetKeyCode()
            if key_code == wx.WXK_RETURN:
                self.__update_sw_image(1)

    def key_released(self, event):
        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            key_code = event.GetKeyCode()
            if key_code == wx.WXK_RETURN:
                self.__update_sw_image(0)

    # Private
    def __setup_menu_bar(self):
        menu = wx.Menu()
        menu.Append(wx.ID_EXIT, "Quit")
        self.menu_bar = wx.MenuBar()
        self.menu_bar.Append(menu, "&File")
        self.SetMenuBar(self.menu_bar)
        self.Bind(wx.EVT_MENU, self.menu_selected)

    def __setup_panels(self):
        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            image = wx.Image("./resources/a640_base.png")
        else:
            image = wx.Image("./resources/a610_base.png")
        board_bitmap = image.ConvertToBitmap()

        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            image = wx.Image("./resources/a640_clear.png")
        else:
            image = wx.Image("./resources/a610_clear.png")
        led_bitmap = image.ConvertToBitmap()
        sw_bitmap = image.ConvertToBitmap()

        self.panel = wx.Panel(self, wx.ID_ANY)
        self.image_panel = wx.Panel(self.panel, wx.ID_ANY)
        self.board_image = wx.StaticBitmap(self.image_panel, wx.ID_ANY, board_bitmap)
        self.led_image = wx.StaticBitmap(self.image_panel, wx.ID_ANY, led_bitmap)
        self.sw_image = wx.StaticBitmap(self.image_panel, wx.ID_ANY, sw_bitmap)

        self.button_panel = wx.Panel(self.panel, wx.ID_ANY)
        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            self.red_button = wx.Button(self.button_panel, id=self.LED_RED_ID, label="RED")
            self.red_button.Bind(wx.EVT_BUTTON, self.button_pressed)
        self.yellow_button = wx.Button(self.button_panel, id=self.LED_YELLOW_ID, label="YELLOW")
        self.yellow_button.Bind(wx.EVT_BUTTON, self.button_pressed)
        
        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            self.red_button.Bind(wx.EVT_KEY_DOWN, lambda event: wx.PostEvent(self.panel, event))
            self.red_button.Bind(wx.EVT_KEY_UP, lambda event: wx.PostEvent(self.panel, event))
        self.yellow_button.Bind(wx.EVT_KEY_DOWN, lambda event: wx.PostEvent(self.panel, event))
        self.yellow_button.Bind(wx.EVT_KEY_UP, lambda event: wx.PostEvent(self.panel, event))
        self.panel.Bind(wx.EVT_KEY_DOWN, self.key_pressed)
        self.panel.Bind(wx.EVT_KEY_UP, self.key_released)
        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            self.red_button.SetFocus()
        else:
            self.yellow_button.SetFocus()

    def __setup_layouts(self):
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.AddStretchSpacer()
        sizer.Add(self.image_panel, flag=wx.ALIGN_CENTER | wx.ALL, proportion=4, border=5)
        sizer.Add(self.button_panel, flag=wx.EXPAND | wx.ALL, proportion=1)
        self.panel.SetSizer(sizer)

        btn_sizer = wx.BoxSizer(wx.HORIZONTAL)
        if self.kotai_id == self.KOTAI_ID_A640_ES or self.kotai_id == self.KOTAI_ID_A640:
            btn_sizer.Add(self.red_button)
        btn_sizer.Add(self.yellow_button)
        btn_base_sizer = wx.BoxSizer(wx.VERTICAL)
        btn_base_sizer.Add(btn_sizer, flag=wx.ALIGN_CENTER)
        self.button_panel.SetSizer(btn_base_sizer)

        self.panel.Layout()

    def __update_led_image(self):
        image_file = self.led_images[self.led_status[self.LED_YELLOW_ID] << 1 | self.led_status[self.LED_RED_ID]]
        image = wx.Image(image_file)
        self.led_image.SetBitmap(image.ConvertToBitmap())

    def __update_sw_image(self, state):
        image_file = self.sw_images[state]
        image = wx.Image(image_file)
        self.sw_image.SetBitmap(image.ConvertToBitmap())
        
class MainController:
    def __init__(self, app):
        kotai_id = self.__get_kotai_id()
        self.view = MainFrame(kotai_id)
        self.view.delegate = self
        led_status = atled.status()
        self.view.set_led_status(led_status)
        self.view.Show()

    # MainFrame delegate method
    def menu_selected(self, view, menu_id):
        if menu_id == wx.ID_EXIT:
            wx.Exit()

    def button_pressed(self, view, id, led_status):
        if id == self.view.LED_RED_ID:
            atled.control(atled.RED, atled.ON if led_status else atled.OFF)
        elif id == self.view.LED_YELLOW_ID:
            atled.control(atled.YELLOW, atled.ON if led_status else atled.OFF)

    def __get_kotai_id(self):
        cmd = "cat /sys/bus/nvmem/devices/imx-ocotp0/nvmem | hexdump -v -s 0xa0 -n 8 -e '/4 \"%08X\"' | cut -c 5-8 | tr -d '\n'"
        proc = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)         
        stdout, _ = proc.communicate()  
        return stdout

if __name__ == "__main__":
    app = wx.App()
    controller = MainController(app)
    app.MainLoop()
